################################################################################
#
#   08_stamAux.R
#
#   This file contains helper methods for StAM.
#
#   written by Claudio Lottaz (COMPDIAG-MPI-MG) in August-October 2003
#
################################################################################
#
#   Access to GO Hashes
#

getGOcategory <- function(ID) GOTERM[[ID]]@Ontology

setMethod("getGOterm", "stamNode",
  function(x) {
    return(GOTERM[[x@ID]]@Term)
  }
)

setMethod("getGOterm", "character",
  function(x) {
    x <- sub("a$", "", x)
    goEntry <- GOTERM[[x]]
    if (!is.null(goEntry)) return(goEntry@Term)
    warning("The GO ID ", x, " not found")
    invisible(NULL)
  }
)

setMethod("getGOchildren", "stamNode",
  function(x) {
    if (class(x) == "stamLeaf") return(character(0))
    if (x@ID == "GO:0003673") return(c("GO:0003674", "GO:0008150", "GO:0005575"))
    if (x@category == "MF") childids <- GOMFCHILDREN[[x@ID]]
    if (x@category == "BP") childids <- GOBPCHILDREN[[x@ID]]
    if (x@category == "CC") childids <- GOCCCHILDREN[[x@ID]]
    if (sum(is.na(childids)) > 0) return(character(0))
    return(childids)
  }
)

setMethod("getGOchildren", "character",
  function(x) {
    if (x == "GO:0003673") return(c("GO:0003674", "GO:0008150", "GO:0005575"))
    gocat <- getGOcategory(x)
    if (gocat == "MF") childids <- GOMFCHILDREN[[x]]
    if (gocat == "BP") childids <- GOBPCHILDREN[[x]]
    if (gocat == "CC") childids <- GOCCCHILDREN[[x]]
    if (sum(is.na(childids)) > 0) return(character(0))
    return(childids)
  }
)

setMethod("getGOparents", "stamNode",
  function(x) {
    if (x@ID == "GO:0003673") return(character(0))
    if (x@category == "MF") parentids <- GOMFPARENTS[[x@ID]]
    if (x@category == "BP") parentids <- GOBPPARENTS[[x@ID]]
    if (x@category == "CC") parentids <- GOCCPARENTS[[x@ID]]
    if (sum(is.na(parentids)) > 0) return(character(0))
    if (length(parentids) == 1) {
      if (as.character(parentids) == "all") return(character(0))
    }
    return(parentids)
  }
)

setMethod("getGOparents", "character",
  function(x) {
    if (x == "GO:0003673") return(character(0))
    gocat <- getGOcategory(x)
    if (gocat == "MF") parentids <- GOMFPARENTS[[x]]
    if (gocat == "BP") parentids <- GOBPPARENTS[[x]]
    if (gocat == "CC") parentids <- GOCCPARENTS[[x]]
    if (sum(is.na(parentids)) > 0) return(character(0))
    if (sum(is.na(parentids)) > 0) return(character(0))
    if (length(parentids) == 1) {
      if (as.character(parentids) == "all") return(character(0))
    }
    return(parentids)
  }
)

################################################################################
#
#   Access to Bioconductor Chip Hashes
#


getprobes <- function(chip="hgu95av2") {
  bimapName <- paste(chip, "GENENAME", sep="")
  genenameBimap <- get(bimapName, paste("package:", chip, ".db", sep=""))
  probeids <- keys(genenameBimap)
  probesidx <- new.env(hash=TRUE)
  for (i in 1:length(probeids)) probesidx[[probeids[i]]] <- i
  return(probesidx)
}

getGOprobes <- function(GOID, chip="hgu95av2") {
  go2probeName  <- paste(chip, "GO2PROBE", sep="")
  go2probeBimap <- get(go2probeName, paste("package:", chip, ".db", sep=""))
  if (is.null(tmp <- go2probeBimap[[GOID]])) return(character(0))
  return(tmp)
}

getprobesymbols <- function(ids, chip="hgu95av2") {
  symbolName  <- paste(chip, "SYMBOL", sep="")
  symbolBimap <- get(symbolName, paste("package:", chip, ".db", sep=""))
  return(unlist(mget(ids, symbolBimap)))
}

getprobenames <- function(ids, chip="hgu95av2") {
  genenameName  <- paste(chip, "GENENAME", sep="")
  genenameBimap <- get(genenameName, paste("package:", chip, ".db", sep=""))
  return(sapply(mget(ids, genenameBimap), paste, collapse=""))
}

################################################################################
#
#   Computing
#

stam.deviance <- function(p, cls, beta) {
  # p: matrix of probabilities, rows are samples, columns classes
  # cls: one class-prediction per sample
  # beta: weighting of classes one weight per class, sum(beta)=1
  d <- 0
  for (i in 1:ncol(p)) {
    index <- cls == colnames(p)[i]
    ni <- sum(index)
    if (ni > 0) d <- d + beta[i] * sum(log(p[index, i])) / ni
  }
  return(-2*d )
}

# variant with log(0) problems
#stam.redundancy <- function(p, nz.nodes=1:dim(p)[3]) {
#  # expects probability matrix p[samples, classes, nodes]
#  # only nodes in nz.nodes are computed
#  # returns a redundancy for each node and an average
#
#  nnodes <- length(nz.nodes)
#  r <- rep(0, nnodes)
#  nz.nodes <- sort(nz.nodes)
#  for (i in 1:(nnodes-1)) {
#    for (j in (i+1):nnodes) {
#      tmp <- -sum(log(p[,,nz.nodes[i]] * (1 - p[,,nz.nodes[j]])))
#      r[i] <- r[i] + tmp
#      r[j] <- r[j] + tmp
#    }
#  }
#  r <- r / (nnodes - 1) / dim(p)[1] / dim(p)[2]
#  return(list(r = r, avg = sum(r)/nnodes))
#}


# improved attampt (avoid log(0))
stam.redundancy <- function(p, nz.nodes=1:dim(p)[3]) {
  # expects probability matrix p[samples, clysses, nodes]
  # only nodes in nz.nodes are computed
  # returns a redundancy for each node and an average

  pdiff <- function(p) 1 - sum(p[,1] * p[,2])

  nnodes <- length(nz.nodes)
  r <- rep(0, nnodes)
  nz.nodes <- sort(nz.nodes)
  for (i in 1:(nnodes-1)) {
    for (j in (i+1):nnodes) {
      tmp <- sum(log(apply(p[,,c(nz.nodes[i],nz.nodes[j])], 1, pdiff)))
      r[i] <- r[i] - tmp
      r[j] <- r[j] - tmp
    }
  }
  r <- r / (nnodes - 1) / dim(p)[1]
  return(list(r = r, avg = sum(r)/nnodes))
}

stam.preds <- function(probs, class.labels) {
  preds.aux <- function(v, cl) {
    p <- which(v == max(v))
    if (length(p) == 1) return(cl[p])
    return("ambiguous")
  }
  preds <- matrix(0, nrow=dim(probs)[1], ncol=dim(probs)[3])
  for (n in 1:dim(probs)[3])
    preds[,n] <- apply(probs[,,n], 1, preds.aux, class.labels)
  return(preds)
}

stam.maxpreds <- function(probs) {
  # determine max overall predictors 
  # expects a matrix of predicted probabilities
  # (samples,classes,nodes), returns for each
  # sample which class obtained the highest
  # maximum probability over all nodes.

  maxpreds.aux <- function(x) {
    m <- which(x==max(x))
    p <- rownames(x)[unique((m - 1) %% nrow(x)) + 1]
    if (length(p) > 1) return (c("ambigous", "-"))
    s <- colnames(x)[m %/% nrow(x)]
    return(c(p, toString(s)))
  }
  predicts <- t(apply(probs, 1, maxpreds.aux))
  colnames(predicts) <- c("Class","Strongest Support")
  rownames(predicts) <- dimnames(probs)[[1]]
  return(predicts)
}
  
stam.threshpreds <- function(probs, cls=NULL, gamma=NULL) {
  # For each class determines max predicted probability
  # over all nodes. if cls is given, determines over all 
  # samples threshold such that lowest error-rate for that 
  # class is reached. Otherwise, gamma must be
  # be specified and predictions are returned.
  # expects a matrix of predicted probabilities
  # (samples,classes,nodes), and a class label
  # for each sample or a threshold for each class.
  # returns for each sample, which class is
  # predicted and for each class the determined 
  # threshold.

  pred <- rep("", dim(probs)[1])
  if (is.null(gamma)) gamma <- numeric(dim(probs)[2])
  for (i in 1:dim(probs)[2]) {
    if (dim(probs)[3] == 1) m <- probs[,i,1]
    else m <- apply(probs[,i,], 1, max)
    if (!is.null(cls)) {
      o <- order(m)
      l <- (cls == dimnames(probs)[[2]][i])[o]
      fp <- 0      # false positives
      fn <- sum(l) # false negatives
      best <- fn + fp
      bestj <- length(l)
      for (j in length(l):2) {
        if (l[j]) fn <- fn-1 else fp <- fp+1
        if (best > fn + fp) { best <- fn + fp; bestj <- j-1 }
      }
      gamma[i] <- m[o[bestj]]
    }
    pred <- paste(pred, ifelse(m > gamma[i], dimnames(probs)[[2]][i], ""), 
                  sep="/")
  }
  pred <- sub("^/*","",pred)
  pred <- sub("/*$","",pred)
  return(list(pred=pred, gamma=gamma))
}

  
stam.sensitivity <- function(preds, sample.classes, class.labels) {
  sens.aux <- function(pr, cl) {
    res <- rep(FALSE, length(pr))
    tmp <- strsplit(pr, "/")
    for (i in 1:length(pr)) res[i] <- cl %in% tmp[[i]]
    return(sum(res)/length(res))
  }
  if (is.matrix(preds)) 
    sens <- matrix(NA, nrow=ncol(preds), ncol=length(class.labels), 
                   dimnames=list(colnames(preds),class.labels))
  else
    sens <- matrix(NA, nrow=1, ncol=length(class.labels), 
                   dimnames=list("",class.labels))
  for (cl in class.labels) {
    select <- sample.classes == cl
    if (sum(select) > 0) {
      if (!is.matrix(preds)) sens[,cl] <- sens.aux(preds[select],cl)
      else sens[,cl] <- apply(preds[select,,drop=FALSE], 2, sens.aux, cl)
    }
  }
  return(sens)
}

stam.specificity <- function(preds, sample.classes, class.labels) {
  spec.aux <- function(pr, cl) {
    res <- rep(FALSE, length(pr))
    tmp <- strsplit(pr, "/")
    for (i in 1:length(pr)) res[i] <- !(cl %in% tmp[[i]])
    return(sum(res)/length(res))
  }

  if (is.matrix(preds)) 
    spec <- matrix(NA, nrow=ncol(preds), ncol=length(class.labels), 
                   dimnames=list(colnames(preds),class.labels))
  else
    spec <- matrix(NA, nrow=1, ncol=length(class.labels), 
                   dimnames=list("",class.labels))
  for (cl in class.labels) {
    select <- sample.classes != cl
    if (sum(select) > 0) {
      if (!is.matrix(preds)) spec[,cl] <- spec.aux(preds[select],cl)
      else spec[,cl] <- apply(preds[select,,drop=FALSE], 2, spec.aux, cl)
                   
    }
  }
  return(spec)
}

stam.listgenes <- function (fit, delta, chip, exprs, classifications) 
{
  y <- factor(classifications[fit$sample.subset])
  x <- exprs[fit$gene.subset, fit$sample.subset]
  geneid <- dimnames(exprs)[[1]][fit$gene.subset]
  nc <- length(unique(y))
  aa <- pamr.predict(fit, x, threshold = delta, type = "nonzero")
  if (length(aa) == 0) return(invisible(NULL))
  cen <- pamr.predict(fit, x, threshold = delta, type = "cen")
  d <- matrix((cen - fit$centroid.overall)[aa, ]/fit$sd[aa], ncol=nc)
  if (is.matrix(d)) oo <- order(-apply(abs(d), 1, max))
  else oo <- c(1)
  d <- round(d, 4)
  g1 <- (geneid[aa])[oo]
  schdr <- paste(dimnames(table(y))$y, "score", sep = " ")
  res <- matrix(d[oo, ], ncol=nc)
  if (dim(res)[1] == 0) return(invisible(NULL))
  res <- cbind(getprobesymbols(g1, chip), getprobenames(g1, chip), res)
  schdr <- c("Symbol", "Genename", schdr)
  dimnames(res) <- list(g1, schdr)
  return(res)
}

stam.nzgraph.rec <- function(nidx, net) {
  # collect infos from node with index nidx
  # store data in object stam.env[["nz"]]
  if (class(net@nodes[[nidx]]) == "stamLeaf") {
    stam.env[["nz"]]$genes <- unique(c(stam.env[["nz"]]$genes, net@nodes[[nidx]]@probes))
  } else {
    nchildren <- length(net@nodes[[nidx]]@children)
    for (c in 1:nchildren) {
      if (net@nodes[[nidx]]@weights[c] > 0) {
        stam.nzgraph.rec(net@nodes[[nidx]]@children[c], net)
      }
    }
  }
  stam.env[["nz"]]$nodes <- unique(c(stam.env[["nz"]]$nodes, nidx))
  invisible(NULL)
}

stam.nzgraph <- function(net) {
  # initialize data to traverse net
  stam.env[["nz"]] <- list(nodes=numeric(0), genes=numeric(0))
  stam.nzgraph.rec(length(net@nodes), net)
  nz <- stam.env[["nz"]]
  rm("nz", envir=stam.env)
  return(nz)
}

################################################################################
#
#   HTML shortcuts
#

# global and generic variables and functions

stam.env[["HTMLfile"]] <- "index.html"
stam.env[["HTMLcon"]]  <- "index.html"
catHTML <- function(...) cat(..., sep="", file=stam.env[["HTMLcon"]])

# HTML initialization, header and footer

beginHTML <- function(title, align="left", outfile="index.html", nocache=FALSE, ...) {
  stam.env[["HTMLfile"]] <- outfile
  stam.env[["HTMLcon"]] <- file(outfile, open="w")
  catHTML("<HTML><HEAD><TITLE>", title, "</TITLE>",
          "<LINK REL='stylesheet' HREF='/includes/style.css' TYPE='text/css'>",
          ifelse(nocache, "<META HTTP-EQUIV='Pragma' CONTENT='no-cache'>", ""),
          "</HEAD><BODY")
  # use additional arguments as attributes for the <BODY>-tag
  addargs <- list(...)
  if (length(addargs) > 0)
    for (i in 1:length(addargs)) catHTML(" ", names(addargs)[i], "='", addargs[[i]], "'")
  catHTML("><H1 align='", align, "'>", title, "</H1>")
}

endHTML <- function() {
  catHTML("<P>&nbsp;</P><HR><ADDRESS>Generated by StAM on ", 
          date(), "</ADDRESS></BODY></HTML>\n")
  close(stam.env[["HTMLcon"]])
}

HTMLtag <- function(tag, ...) {
  # uses named arguments as attributes for 'tag'
  # unnamed arguments are put within the pair
  buffer <- ""
  contents <- NULL
  elipse <- list(...)
  buffer <- paste(buffer, "<", tag, sep="")
  if (!is.null(names(elipse))) {  
    for (i in 1:length(elipse)) 
      if (names(elipse)[i] != "") {
        buffer <- paste(buffer, " ", names(elipse)[i], "='", elipse[[i]], "'", sep="")
      } else contents <- c(contents, elipse[[i]])
  } else contents <- elipse
  buffer <- paste(buffer, ">", sep="")
  for (i in contents) buffer <- paste(buffer, eval(i), " ", sep="")
  buffer <- sub(" $", "", buffer)
  if (tag != "IMG") buffer <- paste(buffer, "</", tag, ">", sep="")
  return(buffer)
}

# Headings

H1 <- function(...) return(HTMLtag("H1", ...))
H2 <- function(...) return(HTMLtag("H2", ...))
H3 <- function(...) return(HTMLtag("H3", ...))
H4 <- function(...) return(HTMLtag("H4", ...))

# Text and Liss

P  <- function(..., align="justify") return(HTMLtag("P", ..., align=eval(align)))
I. <- function(...) return(HTMLtag("I", ...))  # I. becaus I is taken
B  <- function(...) return(HTMLtag("B", ...))
HR <- function(...) return(HTMLtag("HR", ...))
BR <- function() return("<BR>")

UL <- function(...) return(HTMLtag("UL", ...))
OL <- function(...) return(HTMLtag("OL", ...))
LI <- function(...) return(HTMLtag("LI", ...))
#UL <- function(x,...) 
#  return(gsub("</LI>, <LI>", "</LI><LI>", 
#              paste("<UL>", toString(unlist(lapply(x, LI, ...))), "</UL>", sep="")))
#OL <- function(x, ...) 
#  return(gsub("</LI>, <LI>", "</LI><LI>", 
#              paste("<OL>", toString(unlist(lapply(x, LI, ...))), "</OL>", sep="")))

# Image and Tables

IMG <- function(...) return(HTMLtag("IMG", ...))
MAP <- function(...) return(HTMLtag("MAP", ...))

TABLE <- function(...) return(HTMLtag("TABLE", ...))
TR    <- function(...) return(HTMLtag("TR", ...))
TH    <- function(...) return(HTMLtag("TH", ...))
TD    <- function(...) return(HTMLtag("TD", ...))

tableHTML <- function(x, border=NULL, cellspacing=NULL, cellpadding=NULL, width=NULL, 
                      bgcolor=NULL, fgcolor=NULL, table.align=NULL,
                      cell.align=NULL, cell.valign=NULL, header.fgcolor=NULL,
                      header.bgcolor=NULL, header.align=NULL, header.valign=NULL) {
  
  if (is.null(header.bgcolor)) { hdrbgcol <- "" 
  } else hdrbgcol <- paste(" bgcolor='", header.bgcolor,"'", sep="")
  if (is.null(fgcolor)) {
    TDfgcopen  <- THfgcopen  <- ""
    TDfgcclose <- THfgcclose <- ""
  } else {
    TDfgcopen  <- THfgcopen  <- paste("<FONT color='", fgcolor, "'>", sep="")
    TDfgcclose <- THfgcclose <- "</FONT>"
  }
  if (!is.null(header.fgcolor)) {
    THfgcopen  <- paste("<FONT color='", header.fgcolor, "'>", sep="")
    THfgcclose <- "</FONT>"
  }
  if (is.null(header.align )) { hdralign <- "" 
  } else hdralign <- paste(" align='",  header.align,  "'", sep="")
  TDopen  <- paste("<TD>", TDfgcopen,   sep="")
  TDclose <- paste(TDfgcclose, "</TD>", sep="")
  THopen  <- paste("<TH", hdralign, hdrbgcol, ">", THfgcopen, sep="")
  THclose <- paste(THfgcclose, "</TH>", sep="")

  buffer <- "<TABLE"
  if (!is.null(border     )) buffer <- paste(buffer, " border='",      border,      "'", sep="")
  if (!is.null(cellspacing)) buffer <- paste(buffer, " cellspacing='", cellspacing, "'", sep="")
  if (!is.null(cellpadding)) buffer <- paste(buffer, " cellpadding='", cellpadding, "'", sep="")
  if (!is.null(width      )) buffer <- paste(buffer, " width='",       width,       "'", sep="")
  if (!is.null(table.align)) buffer <- paste(buffer, " align='",       table.align, "'", sep="")
  if (!is.null(bgcolor    )) buffer <- paste(buffer, " bgcolor='",     bgcolor,     "'", sep="")
  if (!is.null(fgcolor    )) buffer <- paste(buffer, " fgcolor='",     fgcolor,   "'", sep="")
  buffer <- paste(buffer, ">", sep="")
  if (!is.null(colnames(x))) {
    buffer <- paste(buffer, "<TR", sep="")
    if (!is.null(header.valign)) buffer <- paste(buffer, " valign='", header.valign, "'", sep="")
    buffer <- paste(buffer, ">", sep="")    
    if (!is.null(rownames(x))) buffer <- paste(buffer, THopen, "&nbsp;", THclose, sep="")
    for (col in 1:ncol(x)) 
      buffer <- paste(buffer, THopen, colnames(x)[col], THclose, sep="")
    buffer <- paste(buffer, "</TR>", sep="")
  }
  for (row in 1:nrow(x)) {
    buffer <- paste(buffer, "<TR", sep="")
    if (!is.null(cell.align )) buffer <- paste(buffer, " align='",  cell.align, "'", sep="")
    if (!is.null(cell.valign)) buffer <- paste(buffer, " valign='", cell.valign,     "'", sep="")
    buffer <- paste(buffer, ">", sep="")    
    if (!is.null(rownames(x))) buffer <- paste(buffer, THopen, rownames(x)[row], THclose, sep="")
    for (col in 1:ncol(x)) 
      buffer <- paste(buffer, TDopen, x[row, col], TDclose, sep="")
    buffer <- paste(buffer, "</TR>", sep="")
  }
  buffer <- paste(buffer, "</TABLE>", sep="")
  return(buffer)
}

# Names and References

NAME <- function(label) paste("<A NAME='", label, "'></A>", sep="")
HREF <- function(..., url, target=NULL) {
  if (is.null(target)) { paste("<A HREF=\"", url, "\">", ..., "</A>", sep="")
  } else paste("<A HREF=\"", url, "\" TARGET=\"", target, "\">", ..., "</A>", sep="")
}
HREFannotation <- function(..., url) {
  paste("<A HREF=\"", url, "\" TARGET=\"annotations\">", ..., "</A>", sep="")
}


AFFYREF <- function(ID) HREFannotation(ID, url=getURL(aafProbe(ID)))

GOREF <- function(ID) {
  g <- list()
  class(g) <- "aafGOItem"
  g@id <- ID
  HREFannotation(ID, url=getURL(g))
}

# Forms

FORM <- function(name, action, ..., method = "POST") {
  return(HTMLtag("FORM", NAME=name, ACTION=action, METHOD=method, ...))
}

TEXTFIELD <- function(name, value="", size=10) {
  return(paste("<INPUT TYPE='TEXT' name='",name, "' VALUE='",value, "' SIZE=",size, ">", 
               sep=""))
}

HIDDEN <- function(name, value) {
  return(paste("<INPUT TYPE='HIDDEN' NAME='",name, "' VALUE='",value, "'>", sep=""))
}

PASSWORD <- function(name, value="", size=10) {
  return(paste("<INPUT TYPE=PASSWORD name='",name, "' VALUE='",value, "' SIZE=",size,">",
               sep=""))
}

TEXTAREA <- function(name, ..., rows=4, cols=20) {
   return(HTMLtag("TEXTAEA", NAME=name, ROWS=rows, COLS=cols, ...))
}

CHECKBOX <- function(name, value="ON", checked=FALSE) {
  return(paste("<INPUT TYPE=CHECKBOX NAME='", name, "' VALUE='", value, 
               ifelse(checked, "' CHECKED>", "'>"), sep=""))
}

RADIOBOX <- function(name, value, checked=FALSE) {
  return(paste("<INPUT TYPE=RADIO VALUE='", value, "' NAME='", name, 
               ifelse(checked, "' CHECKED>", "'>"), sep=""))
}

SELECTMENU <- function(name, options, selected=1) {
  tmp <- paste("<SELECT NAME=", name, ">", sep="")
  for (i in 1:length(options)) {
    tmp <- paste(tmp, "<OPTION", ifelse(i==selected, " SELECTED>", ">"),
                 options[i], "</OPTIONS>", sep="")
  }
  return(paste(tmp, "</SELECT>", sep=""))
}

BUTTON <- function(name, value="pushed") {
  return(paste("<INPUT TYPE=BUTTON NAME=", name, " VALUE=", value, ">", sep=""))
}

SUBMIT <- function(name="Submit", value="Submit") {
  return(paste("<INPUT TYPE=SUBMIT VALUE='", value, "' NAME='", name, "'>", sep=""))
}

RESET <- function(name="Rset", value="Reset") {
  return(paste("<INPUT TYPE=RESET VALUE='", value, "' NAME='", name, "'>", sep=""))
}


################################################################################
#
#   Odds and Ends
#

stam.opendev <- function(outfile, ext, ps, res, pointsize, width=9, height=6) {
  if (is.null(outfile)) {
    if (getOption("interactiveStam") && capabilities()["X11"]) 
      X11(width=width, height=height, pointsize=pointsize)
  } else {
    outfile <- paste(outfile, ext, sep="")
    if (ps) postscript(file=paste(outfile, ".ps", sep=""), horizontal=TRUE,
                       paper="special", width=width, height=height, pointsize=pointsize)
    else {
      outfile <- gsub(">", "\\\\>", outfile)
      outfile <- gsub("<", "\\\\<", outfile)
      bitmap(file=paste(outfile, ".png", sep=""),
             width=width, height=height, pointsize=pointsize, res=res)
    }
  }
}

stam.closedev <- function(outfile) {
  if (is.null(outfile) && (!getOption("interactiveStam") || !capabilities()["X11"])) return()
  dev.off()
}

stam.getidx <- function(ids, env) {
  getidx <- function(x, env) if (exists(x, env)) get(x, env) else numeric(0)
  return(unlist(lapply(ids, getidx, env)))
}

# the following two function are taken from pamr which does no longer export them
stam.balanced.folds <- function (y, nfolds = min(min(table(y)), 10))
{
    totals <- table(y)
    fmax <- max(totals)
    nfolds <- min(nfolds, fmax)
    folds <- as.list(seq(nfolds))
    yids <- split(seq(y), y)
    bigmat <- matrix(NA, ceiling(fmax/nfolds) * nfolds, length(totals))
    for (i in seq(totals)) {
        bigmat[seq(totals[i]), i] <- sample(yids[[i]])
    }
    smallmat <- matrix(bigmat, nrow = nfolds)
    smallmat <- stam.permute.rows(t(smallmat))
    res <- vector("list", nfolds)
    for (j in 1:nfolds) {
        jj <- !is.na(smallmat[, j])
        res[[j]] <- smallmat[jj, j]
    }
    return(res)
}

stam.permute.rows <- function (x)
{
    dd <- dim(x)
    n <- dd[1]
    p <- dd[2]
    mm <- runif(length(x)) + rep(seq(n) * 10, rep(p, n))
    matrix(t(x)[order(mm)], n, p, byrow = TRUE)
}

#
#   End of File
#
################################################################################
