\name{image.stamPrediction}
\alias{image.stamPrediction}
\title{Molecular Symptoms Image on StAM Prediction}
\description{
  Shows prediction results for each sample of the class of interest and for all nodes of 
  a StAM model fit as a color coded image.
}
\usage{
\method{image}{stamPrediction}(x, aclass = NULL, main = NULL, 
                     xlab = NULL, show.graph = TRUE, 
                     max.label.length = 40, 
                     sample.labels = FALSE, full.names = TRUE, 
                     outfile = NULL, ps = FALSE, res = 72, 
                     pointsize = 10, width = 11, 
                     minspec = NULL, minsens = 0.1, 
                     maxsens = 1, invert = FALSE, 
                     col = stam.rgb.colors(r0 = 0), 
                     bin.thresh = NULL, ...)
}
\arguments{
  \item{x}{stamPrediction object to be illustrated}
  \item{aclass}{the name of the phenotype class of interest, usually 
    the disease class. If set to NULL the first class in lexicographical order is chosen}
  \item{main}{ the main title of the plot, generated automatically if 
    left blank}
  \item{xlab}{ the label of the x-axis}
  \item{show.graph}{(default is T) whether or not to show GO relations between the nodes}
  \item{max.label.length}{the maximum string length for a GO term}
  \item{sample.labels}{whether or not to show sample names on the x-axis}
  \item{full.names}{whether or not to show GO terms instead of GO IDs}
  \item{outfile}{name of output file if postscript or PNG graphics is 
    to be generated. The extension of the file is chosen automatically according to the 
    \code{ps} argument. If no outfile is specified, an interactive plot is attempted}
  \item{res}{resolution in points per inch}
  \item{ps}{if set to TRUE postscript output is generated}
  \item{pointsize}{the standard fontsize}
  \item{width}{width of image in inches (the height is computed according
    to the number of nodes to be shown and the \code{pointsize})}
  \item{minspec}{nodes to be shown must be at least this specific. If 
    set to NULL this is chosen such that no more than 50 nodes are shown.}
  \item{minsens}{nodes to be shown must be at least this sensitive}
  \item{maxsens}{nodes to be shown must be at most this sensitive}
  \item{invert}{whether or not to invert background and foreground colors}
  \item{col}{the color gradient to code classifier output}
  \item{bin.thresh}{threshold for binary color coding, if left at the 
    default NULL, classifier outputs are coded on a continuous scale.}
  \item{...}{additional options passed to \code{image}}
}
\details{
  This image illustrates classifier outputs generated during a structured analysis of 
  microarrays (StAM). The centralpart of the image shows the color coded classifier 
  outputs for each sample in the class of interest (columns) and each nodes from StAM's 
  model fit (rows). Nodes can be restricted to those of minimal sensitivity or 
  specificity. Also nodes with particularly high sensitivity can be excluded. The color 
  code for the classifier outputs is shown in a color bar on the right hand side if a 
  continuous is used. If a testset is specified only these samples are used to compute 
  sensitivity and specificity. If the sample names are not displayed on the x-axis, the
  test samples are marked with capital letters or just vertical bars (bars if there are 
  too many test samples). Sensitivity and specificity is shown to the left of the figure 
  together with the relations between the GO nodes from the Gene Ontology. GO terms are 
  printed on the right of the image and may be used for a clickable map in HTML.
}
\value{
  A string to be used on an HTML page to provide a clickable map for the GO terms.
}
\author{Claudio Lottaz}
\seealso{\code{\link{stam.predict}}}
\keyword{hplot}
